﻿'***************************************************************************************************
'   入力コントロールのダブルクリック抑制クラス                   clsOmitDoubleClick2(Class)
'
'   ※ボタン等でダブルクリックしてしまうとClickイベントが2回走ってしまうことを抑制するクラス
'     ⇒本クラスはフォームのNewプロシージャ又はLoadイベントで実装すること(Newを推奨)
' 　　⇒フォーム上の各イベントで｢CheckDoubleClick｣を呼ぶ時は本クラスは初期化されているかを判定すること
' 　　　デザイナでのプロパティ設定がNewプロシージャの前にイベントを引き起こすことがあります。
'     ⇒フォーム側Loadイベントで呼び出される場合でも本クラス側でForm_Loadイベントを生成するのは
'     　クラスが解放されずに再度フォーム側Loadイベントが呼び出されるケースに対応するもの
'     　(単独フォームであり得なくてもMDI子フォームではあり得るようです：ガベージコレクタ？)
'     ⇒未更新終了チェック関連クラスは｢clsCheckNotUpdate3｣を使用すること
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 17/01/29(1.0.1.0)新規作成
' 17/10/13(1.0.2.0)Newプロシージャでの初期化対応、blnUseClosing、blnEnabledClassの追加
' 17/10/14(1.0.2.0)g_blnClosingスイッチの追加
' 17/10/16(1.0.2.0)Form_Loadイベントで各モジュールレベル変数の初期化を追加
' 17/11/11(1.0.3.0)未更新終了チェック関連クラスの注意をコメントに追記
' 18/02/20(1.0.3.0)g_blnFirstShownの初期化をNewに移動
'***************************************************************************************************
Friend Class clsOmitDoubleClick2
    '===============================================================================================
    Private g_objOwnerForm As Form = Nothing                ' 呼び出しフォーム
    Private g_blnEnableAppIdle As Boolean = False           ' Application.Idleイベント有効判定
    '-----------------------------------------------------------------------------------------------
    ' 呼び出しフォームとのプロパティ受け渡し変数
    Private g_blnShown As Boolean = False                   ' 初期表示完了スイッチ
    Private g_blnFirstShown As Boolean = True               ' 初回表示判定スイッチ
    Private g_blnOmitDoubleClick As Boolean = False         ' ダブルクリック抑制スイッチ
    Private g_blnClosing As Boolean = False                 ' Closing判定スイッチ

    '***********************************************************************************************
    ' ■■■ クラス初期化／終了メソッド ■■■
    '***********************************************************************************************
    '* 処理名　：New
    '* 機能　　：初期化
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = Ownerフォーム(Object)
    '* 　　　　　Arg2 = FormClosing使用(Boolean)               ※Option(デフォルト=True)
    '* 　　　　　Arg3 = 本クラスの初期化判定(Boolean)          ※Ref参照：Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年01月29日
    '* 作成者　：井上　治
    '* 更新日　：2018年02月20日
    '* 更新者　：井上　治
    '* 機能説明：本クラスで使用するフォーム側イベントの関連付けを生成する
    '* 注意事項：親フォーム側でクラス稼働を知るためにはblnEnabledClassから受け取った変数を参照する
    '***********************************************************************************************
    Friend Sub New(ByRef objOwner As Form, _
                   Optional ByVal blnUseClosing As Boolean = True, _
                   Optional ByRef blnEnabledClass As Boolean = False)
        '-------------------------------------------------------------------------------------------
        g_objOwnerForm = objOwner                   ' Ownerフォーム
        '-------------------------------------------------------------------------------------------
        ' フォームイベント関連付け
        AddHandler g_objOwnerForm.Load, AddressOf Form_Load
        AddHandler g_objOwnerForm.FormClosed, AddressOf Form_FormClosed
        ' FormClosing使用の場合はFormClosingイベント関連付け
        If blnUseClosing Then
            AddHandler g_objOwnerForm.FormClosing, AddressOf Form_FormClosing
        End If
        ' アプリケーションアイドルイベントの関連付け
        AddHandler Application.Idle, AddressOf Application_Idle
        g_blnEnableAppIdle = True
        ' 初回表示判定スイッチを初期化
        g_blnFirstShown = True
        ' 本クラスの初期化完了判定(親フォームへの通知用)
        blnEnabledClass = True
    End Sub

    '***********************************************************************************************
    '* 処理名　：Finalize
    '* 機能　　：クラス終了
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月13日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月13日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Protected Overrides Sub Finalize()
        '-------------------------------------------------------------------------------------------
        ' アプリケーションアイドルイベントの関連付け解除(確認：既に解除されていなかったら)
        If g_blnEnableAppIdle Then
            RemoveHandler Application.Idle, AddressOf Application_Idle
            g_blnEnableAppIdle = False
        End If
        ' 本クラスの解放
        MyBase.Finalize()
    End Sub

    '***********************************************************************************************
    ' ■■■ アプリケーションイベント ■■■
    '***********************************************************************************************
    '* 処理名　：Application_Idle
    '* 機能　　：アプリケーションアイドルイベント
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年01月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月13日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Application_Idle(ByVal sender As Object, ByVal e As System.EventArgs)
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック抑制スイッチのクリア
        If g_blnOmitDoubleClick Then
            g_blnOmitDoubleClick = False
        End If
    End Sub

    '***********************************************************************************************
    ' ■■■ フォームイベント ■■■
    '***********************************************************************************************
    '* 処理名　：Form_FormClosed
    '* 機能　　：フォームイベント(FormClosed)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年01月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月13日
    '* 更新者　：井上　治
    '* 機能説明：アイドルイベントの関連付け解除を行なう
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Form_FormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs)
        '-------------------------------------------------------------------------------------------
        ' アプリケーションアイドルイベントの関連付け解除(確認：既に解除されていなかったら)
        If g_blnEnableAppIdle Then
            RemoveHandler Application.Idle, AddressOf Application_Idle
            g_blnEnableAppIdle = False
        End If
    End Sub

    '***********************************************************************************************
    '* 処理名　：Form_FormClosing
    '* 機能　　：フォームイベント(FormClosing)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月14日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月14日
    '* 更新者　：井上　治
    '* 機能説明：閉鎖動作(Close)に入ったかの判定
    '* 注意事項：親フォーム側の本イベントでCancelがある場合は本スイッチもFalseに戻すこと
    '***********************************************************************************************
    Private Sub Form_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs)
        '---------------------------------------------------------------------------------------
        ' 閉鎖動作(Close)に入った
        g_blnClosing = True
    End Sub

    '***********************************************************************************************
    '* 処理名　：Form_Load
    '* 機能　　：フォームイベント(Load)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年10月13日
    '* 作成者　：井上　治
    '* 更新日　：2018年02月20日
    '* 更新者　：井上　治
    '* 機能説明：アイドルイベントの関連付けを行なう
    '* 注意事項：初期化(New)自体か初期化(New)を通らずに再度Form_Loadが呼び出されるケースの対応
    '***********************************************************************************************
    Private Sub Form_Load(ByVal sender As Object, ByVal e As System.EventArgs)
        '-------------------------------------------------------------------------------------------
        ' アプリケーションアイドルイベントの関連付け(確認：New以後に解除されていたら)
        If Not g_blnEnableAppIdle Then
            AddHandler Application.Idle, AddressOf Application_Idle
            g_blnEnableAppIdle = True
        End If
        ' 各モジュールレベル変数の初期化
        g_blnOmitDoubleClick = False
        g_blnShown = False
        g_blnClosing = False
    End Sub

    '***********************************************************************************************
    ' ■■■ フォームからの呼び出し処理 ■■■
    '***********************************************************************************************
    '* 処理名　：CheckDoubleClick
    '* 機能　　：ダブルクリック等の多重操作の判定とスイッチセット
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：判定結果(Boolean)                           ※True=多重操作orバッチ処理中
    '* 引数　　：Arg1 = スイッチセットのオミット(Boolean)    ※True=Omit(Option)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年01月29日
    '* 作成者　：井上　治
    '* 更新日　：2017年10月14日
    '* 更新者　：井上　治
    '* 機能説明：引数をTrueにするとダブルクリック抑制スイッチのセットは行なわない(判定のみ)
    '* 注意事項：デザイナ設定からフォーム初期化前にコントロールイベントが呼び出されることがある
    '***********************************************************************************************
    Friend Function CheckDoubleClick(Optional ByVal blnOmitSet As Boolean = False) As Boolean
        '-------------------------------------------------------------------------------------------
        CheckDoubleClick = False
        ' ダブルクリック等の多重操作(初期表示未完了、閉鎖動作中も)の判定
        If g_blnOmitDoubleClick OrElse Not g_blnShown OrElse g_blnClosing Then Return True
        ' ※g_swOmitDoubleClickはTrueのまま放置で良い(Application_Idleで解放される)
        If Not blnOmitSet Then
            g_blnOmitDoubleClick = True
        End If
    End Function

    '***********************************************************************************************
    '   プロパティ
    '***********************************************************************************************
    ' 初期表示完了スイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend Property Shown As Boolean
        Set(value As Boolean)
            g_blnShown = value
            ' 閉鎖動作(Close)クリア
            If g_blnShown Then
                g_blnClosing = False
            End If
        End Set
        Get
            Return g_blnShown
        End Get
    End Property

    '===============================================================================================
    ' 初回表示判定スイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend Property FirstShown As Boolean
        Set(value As Boolean)
            g_blnFirstShown = value
        End Set
        Get
            Return g_blnFirstShown
        End Get
    End Property

    '===============================================================================================
    ' ダブルクリック抑制スイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend Property OmitDoubleClick As Boolean
        Set(value As Boolean)
            g_blnOmitDoubleClick = value
        End Set
        Get
            Return g_blnOmitDoubleClick
        End Get
    End Property

    '===============================================================================================
    ' Closing判定スイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend Property Closing As Boolean
        Set(value As Boolean)
            g_blnClosing = value
        End Set
        Get
            Return g_blnClosing
        End Get
    End Property

    '----------------------------------------<< End of Source >>------------------------------------
End Class
